@@ -1,14 +1,14 @@
 deploy = '''
 from setux.targets import Local
 
-exported = Local(name='remote')
-exported.modules.add({exported})
-exported.deploy('{module}', {kwargs})
+host = Local(name='remote')
+host.deploy('{module}'{kwargs})
 '''
 
 script = '''
 #!/bin/bash
 
+export PYTHONPATH={path}:$PYTHONPATH
 cd {path}
 python3 {name}
 '''
