@@ -1,10 +1,11 @@
-from tempfile import NamedTemporaryFile
-from os.path import isdir, basename
 from importlib import import_module
+from os.path import isdir, basename
 from pathlib import Path
+from tempfile import NamedTemporaryFile
 
 from setux.core.errors import ExecError
-from setux.core.target import Target
+from .base import BaseTarget
+from .local import Local
 from . import logger, info, error
 from . import remote_tpl
 
@@ -12,7 +13,7 @@ from . import remote_tpl
 # pylint: disable=arguments-differ
 
 
-class SSH(Target):
+class SSH(BaseTarget):
     def __init__(self, **kw):
         self.host = kw.pop('host', None)
         self.priv = kw.pop('priv', None)
@@ -22,6 +23,10 @@ class SSH(Target):
         kw['name'] = kw.pop('name', self.host)
         super().__init__(**kw)
 
+    def set_local(self):
+        self.local = Local()
+        return self.local
+
     def skip(self, line):
         if (
             line.startswith('Connection to')
@@ -36,12 +41,25 @@ class SSH(Target):
                 a = a.replace("'", r"'\''")
                 a = f"'{a}'"
                 arg[i] = a
+            elif '*' in a:
+                a = f'"{a}"'
+                arg[i] = a
+            elif any(c in a for c in '|><'):
+                a = f"'{a}'"
+                arg[i] = a
         command = ' '.join(arg)
         check = kw.pop('check', False)
         cmd = ['ssh']
         if self.priv: cmd.extend(['-i', self.priv])
         cmd.append(self.host)
         term = kw.pop('term', False)
+        if sudo := kw.pop('sudo', None):
+            try:
+                login = self.distro.login.name
+                if sudo != login:
+                    term = True
+                    arg = ['sudo', f'--user={sudo}'] + arg
+            except Exception: pass
         cmd.append('-t' if term else '-T')
         cmd.extend(arg)
         kw['skip'] = self.skip
@@ -51,11 +69,6 @@ class SSH(Target):
             raise ExecError(command, ret, out, err)
         return ret, out, err
 
-    def __call__(self, command, **kw):
-        ret, out, err = self.run(command, **kw)
-        info('\n\t'.join(out))
-        return ret
-
     def chk_cnx(self, report='quiet'):
         ret, out, err = self.run('uname', report='quiet')
         if ret == 0:
@@ -79,18 +92,24 @@ class SSH(Target):
         self.trace('scp '+' '.join(arg), ret, out, err, **kw)
         return ret, out, err
 
-    def send(self, local, remote=None, quiet=False):
-        local =  Path(local)
-        assert local.is_file()
-        remote = Path(remote) if remote else local
-        if not quiet: info(f'\tsend {local} -> {remote}')
-        ret, out, err = self.scp(f'{local} {self.host}:{remote}')
-        return ret==0
-
-    def fetch(self, remote, local, quiet=False):
-        if not quiet: info(f'\tfetch {local} <- {remote}')
+    def do_fetch(self, remote, local, quiet=False):
         ret, out, err = self.scp(f'{self.host}:{remote} {local}')
-        return ret==0
+        return ret == 0
+
+    def do_fetch_as(self, sudo, remote, local, quiet=False):
+        tmp = Path('/tmp/setux/') / Path(local)
+        login = self.distro.login.name
+        ret, out, err = self.run(f'mkdir -p {tmp.parent}')
+        ok = ret == 0
+        ret, out, err = self.run(f'cp {remote} {tmp}', sudo=sudo)
+        ok = ok and ret == 0
+        ret, out, err = self.run(f'chown {login} {tmp}', sudo=sudo)
+        ok = ok and ret == 0
+        ret, out, err = self.scp(f'{self.host}:{tmp} {local}')
+        ok = ok and ret == 0
+        ret, out, err = self.run(f'rm {tmp}')
+        ok = ok and ret == 0
+        return ok
 
     def rsync_opt(self):
         if self.priv:
@@ -98,31 +117,41 @@ class SSH(Target):
         else:
             return '-e ssh'
 
-    def sync(self, src, dst=None):
+    def do_sync(self, src, dst=None):
         assert isdir(src), f'\n ! sync reqires a dir ! {src} !\n'
         if not src.endswith('/'): src+='/'
         if not dst: dst = src
         info(f'\tsync {src} -> {dst}')
         return self.rsync(f'{src} {self.host}:{dst}')
 
-    def read(self, path, mode='rt', critical=True, report='normal'):
-        if report=='normal':
-            info(f'\tread {path}')
-        with NamedTemporaryFile(mode=mode) as tmp:
-            self.fetch(path, tmp.name, quiet=True)
-            content = tmp.read()
-        return content
+    def do_send(self, local, remote=None):
+        assert Path(local).is_file()
+        remote = remote or local
+        dst = Path(remote).parent
+        ret, out, err = self.run(f'mkdir -p {dst}')
+        if ret: return False
+        ret, out, err = self.scp(f'{local} {self.host}:{remote}')
+        return ret == 0
 
-    def write(self, path, content, mode='wt', report='normal'):
-        if report=='normal':
-            info(f'\twrite {path}')
-        dest = path[:path.rfind('/')]
-        self.run(f'mkdir -p {dest}', report=report)
-        with NamedTemporaryFile(mode=mode) as tmp:
-            tmp.write(content)
-            tmp.flush()
-            self.send(tmp.name, path, quiet=True)
-        return self.read(path, mode=mode.replace('w','r'), report='quiet') == content
+    def do_send_as(self, sudo, local, remote):
+        local = Path(local)
+        assert local.is_file()
+        remote = Path(remote)
+        tmp = Path('/tmp/setux'+ str(local))
+        login = self.distro.login.name
+        ret, out, err = self.run(f'mkdir -p {tmp.parent}')
+        ok = ret == 0
+        ret, out, err = self.scp(f'{local} {self.host}:{tmp}')
+        ok = ok and ret == 0
+        ret, out, err = self.run(f'mkdir -p {remote.parent}', sudo=sudo)
+        ok = ok and ret == 0
+        ret, out, err = self.run(f'cp {tmp} {remote}', sudo=sudo)
+        ok = ok and ret == 0
+        ret, out, err = self.run(f'rm {tmp}')
+        ok = ok and ret == 0
+        ret, out, err = self.run(f'chown {sudo} {remote}', sudo=sudo)
+        ok = ok and ret == 0
+        return ok
 
     def export(self, name, root):
         info(f'\texport {name} -> {root}')
